# -*- coding: utf-8; -*-
################################################################################
#
#  WuttaSync -- Wutta Framework for data import/export and real-time sync
#  Copyright © 2024 Lance Edgar
#
#  This file is part of Wutta Framework.
#
#  Wutta Framework is free software: you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option) any
#  later version.
#
#  Wutta Framework is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#  more details.
#
#  You should have received a copy of the GNU General Public License along with
#  Wutta Framework.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
``wutta import-csv`` command
"""

import inspect
from pathlib import Path
from typing import List, Optional
from typing_extensions import Annotated

import makefun
import typer


def importer_command_template(

        # model keys
        models: Annotated[
            Optional[List[str]],
            typer.Argument(help="Model(s) to process.  Can specify one or more, "
                           "or omit to process all default models.")] = None,

        # allow create?
        create: Annotated[
            bool,
            typer.Option(help="Allow new target records to be created.")] = True,

        # allow update?
        update: Annotated[
            bool,
            typer.Option(help="Allow existing target records to be updated.")] = True,

        # allow delete?
        delete: Annotated[
            bool,
            typer.Option(help="Allow existing target records to be deleted.")] = False,

        # fields
        fields: Annotated[
            str,
            typer.Option('--fields',
                         help="List of fields to process.  See also --exclude.")] = None,
        excluded_fields: Annotated[
            str,
            typer.Option('--exclude',
                         help="List of fields *not* to process.  See also --fields.")] = None,

        # dry run?
        dry_run: Annotated[
            bool,
            typer.Option('--dry-run',
                         help="Go through the motions, but rollback the transaction.")] = False,

):
    """
    Stub function which provides a common param signature; used with
    :func:`importer_command()`.
    """


def importer_command(fn):
    """
    Decorator for import/export commands.  Adds common params based on
    :func:`importer_command_template()`.

    To use this, e.g. for ``poser import-foo`` command::

       from poser.cli import poser_typer
       from wuttasync.cli import importer_command

       @poser_typer.command()
       @importer_command
       def import_foo(
               ctx: typer.Context,
               **kwargs
       ):
           \"""
           Import data from Foo API to Poser DB
           \"""
           from poser.importing.foo import FromFooToPoser

           config = ctx.parent.wutta_config
           kw = dict(ctx.params)
           models = kw.pop('models')
           handler = FromFooToPoser(config)
           handler.process_data(*models, **kw)
    """
    original_sig = inspect.signature(fn)
    reference_sig = inspect.signature(importer_command_template)

    params = list(original_sig.parameters.values())
    for i, param in enumerate(reference_sig.parameters.values()):
        params.insert(i + 1, param)

    # remove the **kwargs param
    params.pop(-1)

    final_sig = original_sig.replace(parameters=params)
    return makefun.create_function(final_sig, fn)


def file_importer_command_template(
        input_file_path: Annotated[
            Path,
            typer.Option('--input-path',
                         exists=True, file_okay=True, dir_okay=True,
                         help="Path to input file(s).  Can be a folder "
                         "if app logic can guess the filename(s); "
                         "otherwise must be complete file path.")] = ...,
):
    """
    Stub function to provide signature for import/export commands
    which require input file.  Used with
    :func:`file_importer_command()`.
    """


def file_importer_command(fn):
    """
    Decorator for import/export commands which require input file.
    Adds common params based on
    :func:`file_importer_command_template()`.

    To use this, it's the same method as shown for
    :func:`importer_command()` except in this case you would use the
    ``file_importer_command`` decorator.
    """
    original_sig = inspect.signature(fn)
    plain_import_sig = inspect.signature(importer_command_template)
    file_import_sig = inspect.signature(file_importer_command_template)
    desired_params = (
        list(plain_import_sig.parameters.values())
        + list(file_import_sig.parameters.values()))

    params = list(original_sig.parameters.values())
    for i, param in enumerate(desired_params):
        params.insert(i + 1, param)

    # remove the **kwargs param
    params.pop(-1)

    final_sig = original_sig.replace(parameters=params)
    return makefun.create_function(final_sig, fn)
